# eForms-de Schematron

Schematron Rules for XML validation of the German eForms-DE business rules.

## Changes and Semantic Versioning

We use [Semantic Versioning](https://semver.org/spec/v2.0.0.html) as a means to indicate how severe a change might be to your system.

Currently, we are on the Major `0` i.e. any release can break previously valid eForms-DE XML.

Given a version number MAJOR.MINOR.PATCH, we increment the:

1. MAJOR version when we make incompatible changes:
    * That is we introduce changes to the rules which will **DEFINITELY** not validate previously valid XML anymore
1. MINOR version when we add functionality which **MIGHT** not validate previously valid XML anymore
    * Widen the scope of valid content: e.g. we add new codelist terms or accept new optional XML elements or otherwise make existing rules less strict.
        * **CAUTION**: If you use this widened scope the receiving site using older version(s) might not validate
    * Narrow the scope of valid content:  
        * If we narrow it in order to be compliant to existing specification (i.e. in order to fix false positive validation), the effect dependents on your system.
1. PATCH version when we make backwards-compatible bug fixes or other enhancements not changing intended validation semantics.

## Examples

Valid examples for all notice subtypes in eForms-DE format can be found [here](/src/test/eforms-de-all-notices-examples/)

## Contact/Support

Existing issues can be found at [our issue tracker](https://projekte.kosit.org/eforms/eforms-de-schematron/-/issues).

If you think you have found a bug, please [contact us](https://xeinkauf.de/kontakt/#support).

<!-- 
You can find packaged releases on [our GitLab project](https://projekte.kosit.org/xrechnung/xrechnung-schematron/-/releases).
-->

## Contributing

Any contribution in terms of Feedback, opened Issues and Merge Requests are always welcome.

## Development

Further information on development may be found in our [Developer Documentation](./doc/development.md)


## License

Apache 2
