# Project Structure

Schematron sources are in `src/main/schematron/` directory.

## Naming Assertions

We use a semantically loaded scheme:

`CL-DE-BT-${num}` with

* `CL`= Code list
* `DE` = German Alpha-2 ISO Country Code
* `BT-{num} for the Business Term this code list assertion applies to
*

## Testing

We are only testing with **Saxon HE 10.x**.

Proper execution of the rules with other XSLT environments cannot be guaranteed.

## Test case development with XML Mutate

We also create test cases using XML Mutate <https://projekte.kosit.org/kosit/xml-mutate> .

In order to successfully download the xml-mutate dependency, following should be done:
1. Add server configuration to your settings.xml file:
 ```shell
    <servers>
		<!-- You only need this for deploying using ant or -->
		<server>
			<id>xse-component-repository</id>
			<configuration>
				<httpHeaders>
					<property>
						<name>Private-Token</name>
						<value>${A_GITLAB_TOKEN}</value>
					</property>
				</httpHeaders>
			</configuration>
		</server>
	</servers>
```
2. Resolving dependencies is currently a bit more tricky. As of version 1.5 the `settings.xml`` 
   seems to be the only option to let it search for artifacts in Maven Central as well as in 
   this Repository. Maven profile is needed to use multiple repositories. Add profile 
   configuration to your settings.xml:
```shell
    <profiles>
		<profile>
			<id>use-xse-component-repository</id>
			<repositories>
				<repository>
					<id>xse-component-repository</id>
					<name>KoSIT GitLab Maven Repo</name>
					<url>https://projekte.kosit.org/api/v4/projects/356/packages/maven</url>
				</repository>
			</repositories>
		</profile>
	</profiles>
	<!-- Always active-->
	<activeProfiles>
		<activeProfile>use-xse-component-repository</activeProfile>
	</activeProfiles>
```
See https://maven.apache.org/guides/mini/guide-multiple-repositories.html. 

Otherwise, you can manually download the newest version. You have two choices:

1. Copy an `xml-mutate.jar` into `lib` directory
2. If you set the ant property to the URL of directory (full path) like e.g. `xmute.download.url.prefix=file:/mnt/c/data/git-repos/xml-mutator/target` (Linux), you can als execute the `test` target e.g.

```shell
ant -Dxmute.download.url.prefix=file:/home/renzo/projects/xml-mutate/target test
```

For Windows users:

```shell
ant "-Dxmute.download.url.prefix=file:/c:/dev/git/xml-mutate/target" test
```

The `src/test/` directory contains instances for the sole purpose to cover detailed technical aspects of eForms-DE Schematron development such as codelist tests among others. These tests have Unit Test character by focusing on testing single Schematron rules in isolation. Hence, these technical cases might be invalid instances w.r.t. to eForms-de specification.

The naming convention for the tests is *format*-*rule*-*description*.xml (e.g. cii-br-de-24-test-bg-17.xml).

### Conventions

In order to keep test case development consistent it is important to follow several conventions.

#### Common names for different Schematron rule sets

The following names are used to reference different Schematron files

```
efde:  for eForms-de rules
```

## The build environment

We recommend `Apache Ant` version 1.10.x or newer (but should work with > 1.8.x).

The main `ant` targets for developing are:

* `compile`
* `test` (unit testing)
* and `dist` (creating the distribution artefact)

However, because of the complex dependencies, you may only expect `compile` target to work without any customizations.

## Distribution

The `ant` target `dist` creates the distribution zip archive.

## Creating a Release

### Checklist


* Are all issues scheduled for the release solved?
* Is everything merged to `main` branch?
* Make sure you committed and pushed everything
* Do Schematron files include correct version of eForms-DE specification?
* ⚠️ Make sure that CHANGELOG.md is up to date
* Make sure to update versions in `pom.xml`

### Prepare

* Create the distribution by **not** using your development properties file.

This requires to set some at command line:

```bash
ant -Dxmute.download.url.prefix='file:/${YOUR_PATH_TO}/xml-mutate/target' clean dist
```

* Tag the last commit according to the following naming rule: `v${eforms-de-schematron.version.full}` e.g.
  `git tag v0.6.0 && git push origin v0.6.0`

### Publish

* Upload the distribution artifact as a generic package to the GitLab Package Registry e.g. using a cli like the following

```bash
curl --header "PRIVATE-TOKEN: ${YOUR_GITLAB_PRIVAT_TOKEN}" --upload-file dist/eforms-de-{eforms-de.version.full}-schematron-{eforms-de-schematron.version.full}.zip      "https://projekte.kosit.org/api/v4/projects/276/packages/generic/eforms-de-schematron/{eforms-de-schematron.version.full}/eforms-de-{eforms-de.version.full}-schematron-{eforms-de-schematron.version.full}.zip"
```

* Draft a new release at [https://projekte.kosit.org/eforms/eforms-de-schematron/-/releases/new](https://projekte.kosit.org/eforms/eforms-de-schematron/-/releases/new)
  * Choose the git tag you just created
* Add release title of the following scheme: `eforms-de-{eforms-de.version.full}-schematron-{eforms-de-schematron.version.full}`
* Copy & paste the high quality changelog entries for this release from CHANGELOG.md.
* Link distribution zip of GitLab Package Registry

* Publish the new release in GitLab

### Post-Release

* Change the version of eForms-DE Schematron in `pom.xml` to the next release and commit

You are done :smile:
